package com.cubetiqs.sql;

import java.util.Properties;

public interface ConnectionProperties {
    String getType();

    String getHost();

    int getPort();

    String getUser();

    String getPassword();

    String getDatabase();

    default boolean useSSL() {
        return false;
    }

    default Properties getProperties() {
        return null;
    }

    default StringBuilder getURLBuilder() {
        return new StringBuilder("jdbc:" + getType() + "://")
                .append(getHost())
                .append(":")
                .append(getPort())
                .append("/")
                .append(getDatabase());
    }

    class MySQL implements ConnectionProperties {
        private String host;
        private int port;
        private String user;
        private String password;
        private String database;
        private boolean useSSL;
        private Properties properties;

        private boolean autoReconnect = true;
        private int cmaxReconnects = 5;
        private int initialTimeout = 1;


        @Override
        public String getType() {
            return "mysql";
        }

        @Override
        public String getHost() {
            return host;
        }

        @Override
        public int getPort() {
            if (port > 0) {
                return port;
            }

            return 3306;
        }

        @Override
        public String getUser() {
            if (user == null || user.equals("")) {
                return "root";
            }

            return user;
        }

        @Override
        public String getPassword() {
            return password;
        }

        @Override
        public String getDatabase() {
            return database;
        }

        @Override
        public boolean useSSL() {
            return useSSL;
        }

        @Override
        public Properties getProperties() {
            return properties;
        }

        @Override
        public StringBuilder getURLBuilder() {
            return ConnectionProperties.super.getURLBuilder()
                    .append("?").append("useSSL=").append(useSSL)
                    .append("&").append("autoReconnect=").append(autoReconnect)
                    .append("&").append("cmaxReconnets=").append(cmaxReconnects)
                    .append("&").append("initialTimeout").append(initialTimeout);
        }
    }
}
