package com.cubetiqs.sql;

import java.util.Objects;

public interface IExecuteResultWithCount<R> extends IExecuteResult<R> {
    long getCount();

    static <R> IExecuteResultWithCount<R> newEmpty() {
        return new IExecuteResultWithCount<R>() {
            @Override
            public long getCount() {
                return 0;
            }

            @Override
            public R getData() {
                return null;
            }
        };
    }

    static <R> IExecuteResultWithCount<R> with(long count, IExecuteResult<R> result) {
        Objects.requireNonNull(result, "IExecuteResult must be not null!");

        return new IExecuteResultWithCount<R>() {
            @Override
            public long getCount() {
                return count;
            }

            @Override
            public R getData() {
                return result.getData();
            }

            @Override
            public long getDuration() {
                return result.getDuration();
            }

            @Override
            public Throwable getError() {
                return result.getError();
            }
        };
    }
}
