package com.cubetiqs.sql;

import com.cubetiqs.sql.mapper.MapperProvider;
import com.cubetiqs.sql.mapper.RowMapperProvider;

import java.util.List;
import java.util.Map;

public interface JdbcDataQuery extends JdbcAccessor, JdbcCountQuery {
    IExecuteResult<List<Object>> queryForList(String sql);

    IExecuteResult<List<Object>> queryForList(String sql, Object... args);

    <R> IExecuteResult<List<R>> queryForList(String sql, RowMapperProvider<R> mapper);

    <R> IExecuteResult<List<R>> queryForList(String sql, RowMapperProvider<R> mapper, Object... args);

    IExecuteResult<Object> queryForObject(String sql);

    IExecuteResult<Object> queryForObject(String sql, Object... args);

    <R> IExecuteResult<R> query(MapperProvider<Map<String, Object>, R> mapper, String sql, Class<R> clazz);

    <R> IExecuteResult<R> query(MapperProvider<Map<String, Object>, R> mapper, String sql, Class<R> clazz, Object... args);

    /**
     * Query the Object "R" and the first row.
     *
     * @param sql    String
     * @param mapper RowMapperProvider
     * @param <R>    Result
     * @return IExecuteResult
     */
    <R> IExecuteResult<R> query(String sql, RowMapperProvider<R> mapper);

    <R> IExecuteResult<R> query(String sql, RowMapperProvider<R> mapper, Object... args);

    default IExecuteResultWithCount<List<Object>> queryOf(String sql) {
        IExecuteResult<List<Object>> result = queryForList(sql);
        long count = 0;
        if (result.getData() != null) {
            count = result.getData().size();
        }

        if (count == 0) {
            return IExecuteResultWithCount.newEmpty();
        }

        return IExecuteResultWithCount.with(count, result);
    }

    default IExecuteResultWithCount<List<Object>> queryOf(String sql, Object... args) {
        IExecuteResult<List<Object>> result = queryForList(sql, args);
        long count = 0;
        if (result.getData() != null) {
            count = result.getData().size();
        }

        if (count == 0) {
            return IExecuteResultWithCount.newEmpty();
        }

        return IExecuteResultWithCount.with(count, result);
    }

    default IExecuteResultWithCount<List<Object>> queryOf(ISqlMapParameter parameter) {
        if (parameter.getParams().size() == 0) {
            return queryOf(parameter.getSql());
        }

        Object[] args = parameter.getSqlArgs();
        return queryOf(parameter.getFormatSql(), args);
    }

    default <R> IExecuteResultWithCount<List<R>> queryOf(String sql, RowMapperProvider<R> mapper) {
        IExecuteResult<List<R>> result = queryForList(sql, mapper);
        long count = 0;
        if (result.getData() != null) {
            count = result.getData().size();
        }

        if (count == 0) {
            return IExecuteResultWithCount.newEmpty();
        }

        return IExecuteResultWithCount.with(count, result);
    }

    default <R> IExecuteResultWithCount<List<R>> queryOf(String sql, RowMapperProvider<R> mapper, Object... args) {
        IExecuteResult<List<R>> result = queryForList(sql, mapper, args);
        long count = 0;
        if (result.getData() != null) {
            count = result.getData().size();
        }

        if (count == 0) {
            return IExecuteResultWithCount.newEmpty();
        }

        return IExecuteResultWithCount.with(count, result);
    }

    default <R> IExecuteResultWithCount<List<R>> queryOf(ISqlMapParameter parameter, RowMapperProvider<R> mapper) {
        if (parameter.getParams().size() == 0) {
            return queryOf(parameter.getSql(), mapper);
        }

        Object[] args = parameter.getSqlArgs();
        return queryOf(parameter.getFormatSql(), mapper, args);
    }

    default IExecuteResult<List<Object>> queryForList(ISqlMapParameter parameter) {
        if (parameter.getParams().size() == 0) {
            return queryForList(parameter.getSql());
        }

        Object[] args = parameter.getSqlArgs();
        return queryForList(parameter.getFormatSql(), args);
    }

    default <R> IExecuteResult<List<R>> queryForList(ISqlMapParameter parameter, RowMapperProvider<R> mapper) {
        if (parameter.getParams().size() == 0) {
            return queryForList(parameter.getSql(), mapper);
        }

        Object[] args = parameter.getSqlArgs();
        return queryForList(parameter.getFormatSql(), mapper, args);
    }

    default IExecuteResult<Object> queryForObject(ISqlMapParameter parameter) {
        if (parameter.getParams().size() == 0) {
            return queryForObject(parameter.getSql());
        }

        Object[] args = parameter.getSqlArgs();
        return queryForObject(parameter.getFormatSql(), args);
    }

    default <R> IExecuteResult<R> query(MapperProvider<Map<String, Object>, R> mapper, ISqlMapParameter parameter, Class<R> clazz) {
        if (parameter.getParams().size() == 0) {
            return query(mapper, parameter.getSql(), clazz);
        }

        Object[] args = parameter.getSqlArgs();
        return query(mapper, parameter.getFormatSql(), clazz, args);
    }

    default <R> IExecuteResult<R> query(ISqlMapParameter parameter, RowMapperProvider<R> mapper) {
        if (parameter.getParams().size() == 0) {
            return query(parameter.getSql(), mapper);
        }

        Object[] args = parameter.getSqlArgs();
        return query(parameter.getFormatSql(), mapper, args);
    }
}
